#include "parser.hpp"

#include <cstring>
#include <algorithm>
#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <stack>

using namespace std;

char _acid[5]={'a','c','g','t','u'};
std::set<char> acid(_acid, _acid+5);
std::string _basepair[10]={"au", "ua", "gc", "cg", "gu", "ug", "at", "ta", "gt", "tg"};
std::set<std::string> basepair(_basepair, _basepair+10);

string getPair(string seq, int i, int j){
    return string(1, seq[i])+string(1, seq[j]);
}

int acid2int(const char c){
    switch(c){
        case '-':
            return 0;
        case 'a':
            return 1;
        case 'c':
            return 2;
        case 'g':
            return 3;
        case 't':
        case 'u':
            return 4;
        default:
            return 0;
    }
}

int BP2int(string bp){
    if(bp.compare("au")==0 || bp.compare("at")==0)
        return 1;
    else if(bp.compare("ua")==0 || bp.compare("ta")==0)
        return 2;
    else if(bp.compare("gu")==0 || bp.compare("gt")==0)
        return 3;
    else if(bp.compare("ug")==0 || bp.compare("tg")==0)
        return 4;
    else if(bp.compare("gc")==0)
        return 5;
    else if(bp.compare("cg")==0)
        return 6;
    else
        return 0;
}

int readCW(string fname, map<string,string>& MSA){
    string name;
    string ws;
    string seq;
    unsigned int seqlen;
    stringstream ss;

    string line;
    ifstream fp(fname.c_str());
    if(fp.is_open()){
        getline(fp, line);
        if(strncmp(line.c_str(), "CLUS", 4)!=0){
            cerr<<"The file "<<fname<<" is not in Clustal W format"<<endl;
            fp.close();
            return 0;
        }
        while(fp.good()){
            getline(fp, line);
            if(isspace(line[0]) || line[0]==0) continue;
            ss.clear();
            ss<<line;
            ss>>name>>seq;
            transform(seq.begin(), seq.end(), seq.begin(), (int(*)(int))tolower);
            MSA[name]+=seq;
        }
        seqlen=(MSA.begin()->second).length();
        for(map<string, string>::iterator i=MSA.begin(); i!=MSA.end(); i++){
            if((i->second).length()!=seqlen){
                cerr<<"The lengthes of sequences in MSA are not the same"<<endl;
                return 0;
            }
        }
        fp.close();
        return 1;
    }
    else{
        cerr<<"Unable to open Clustal W file: "<<fname<<endl;
        return 0;
    }
}

PNode::PNode():sname(""), weight(0.), cost(0), label(){
    lsub=0;
    rsub=0;
}

PNode::PNode(string iname, double iweight):sname(iname), weight(iweight), cost(0), label(){
    lsub=0;
    rsub=0;
}

PNode::~PNode(){
    delete lsub;
    delete rsub;
}

void PNode::show(int indent){
    cout<<setw(indent*2)<<"("<<weight<<","<<cost<<"):"<<sname<<endl;
    if(lsub!=0)
        lsub->show(indent+1);
    if(rsub!=0)
        rsub->show(indent+1);
}

int Newick_parser(string newick, PNode* root){
    stack<PNode*> nstack;
    PNode* p;
    nstack.push(root);

    string sname="";
    double weight=0.;
    string str="";
    for(unsigned int i=0; i<newick.length(); i++){
        if(newick[i]=='('){
            PNode* lsub=new PNode();
            nstack.top()->lsub=lsub;
            nstack.push(lsub);
        }
        else if(newick[i]==':'){
            sname=str;
            str="";
        }
        else if(newick[i]==','){
            if(str.length()>0)
                weight=atof(str.c_str());
            p=nstack.top();
            p->sname=sname;
            p->weight=weight;
            nstack.pop();
            PNode* rsub=new PNode();
            nstack.top()->rsub=rsub;
            nstack.push(rsub);

            sname="";
            weight=0.;
            str="";
        }
        else if(newick[i]==')'){
            if(str.length()>0)
                weight=atof(str.c_str());
            p=nstack.top();
            p->sname=sname;
            p->weight=weight;
            nstack.pop();

            sname="";
            weight=0.;
            str="";
        }
        else if(newick[i]=='\t' || newick[i]==' '){
            continue;
        }
        else if(newick[i]==';'){
            break;
        }
        else{
            str+=newick[i];
        }
    }
    return 1;
}

int readNewick(string fname, PNode* root){
    string newick="";

    string line;
    ifstream fp(fname.c_str());
    if(fp.is_open()){
        while(fp.good()){
            getline(fp, line);
            newick+=line;
        }
    }
    else{
        cerr<<"Unable to open Newick file: "<<fname<<endl;
        return 0;
    }
    return Newick_parser(newick, root);
}
